==> hyades.source_1.0.3_20030929_1406.zip: org/eclipse/hyades/log/ui/internal/views/LogAnalyzerImpl.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.log.ui.internal.views;

import java.util.*;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.emf.common.util.*;
import org.eclipse.hyades.analysis.engine.*;
import org.eclipse.hyades.log.ui.internal.*;
import org.eclipse.hyades.models.cbe.*;
import org.eclipse.hyades.models.internal.sdb.*;
import org.eclipse.jface.viewers.*;


public class LogAnalyzerImpl implements ILogAnalyzer {

	protected String errorMessage;
    protected Map _analysisEngine = new Hashtable();
    private final String PLATFORM_PREFIX = "platform:/resource";	
	/**
	 * Constructor for LogAnalyzerImpl.
	 */
	public LogAnalyzerImpl() {
		super();
	}

	/**
	 * @see org.eclipse.hyades.analysis.engine.ILogAnalyzer#errorMsg()
	 */
	public String errorMsg() {
		return errorMessage;
	}

	/**
	 * @see org.eclipse.hyades.analysis.engine.ILogAnalyzer#loadDatabase()
	 */
	public void loadDatabase() {
		
		errorMessage = null;

		Map list = getSymptomDBPathList();
		
		Set keySet = _analysisEngine.keySet();

		for (Iterator iter = keySet.iterator(); iter.hasNext();) {
			AnalysisEngine element = (AnalysisEngine) iter.next();
			if (!databaseExists(element.getSymptomDatabasePath())) {
				element.removeSymptomDatabase();
				iter.remove();
				continue;
			}			

			String path = element.getSymptomDatabasePath();			
			Long value = (Long)list.get(path);
			if(value==null){
				element.removeSymptomDatabase();
				iter.remove();			
			}else if((value!=null && value.longValue()!=((Long)_analysisEngine.get(element)).longValue())){				
				element.removeSymptomDatabase();
				iter.remove();			
			}else{
				list.remove(path);
			}
		}
		
		Set listSet = list.keySet();
		for (Iterator iter = listSet.iterator(); iter.hasNext();) {
			String path = (String) iter.next();
			_analysisEngine.put(AnalysisEngine.getInstance(path), list.get(path));			
		}
		
		if (_analysisEngine.size() == 0) {

			errorMessage = LogUIPlugin.getResourceString("STR_NO_SYMPTOM_DB_ERROR");

		}    
	}
		
	/**
	 * @see org.eclipse.hyades.analysis.engine.ILogAnalyzer#unloadDatabase()
	 */
	public void unloadDatabase() {

		Set analysisSet = _analysisEngine.keySet();
		for (Iterator iter = analysisSet.iterator(); iter.hasNext();) {
			AnalysisEngine engine = (AnalysisEngine) iter.next();
			engine.removeSymptomDatabase();

		}
		_analysisEngine.clear();			

	}

	protected Map getSymptomDBPathList() {
		Map list = new HashMap();
		String symptomdbPath = AnalysisEnginePlugin.getPlugin().getPreferenceStore().getString(AnalysisEngineConstants.SYMPTOM_DB_PATH);

		if (symptomdbPath == null || symptomdbPath.equals(""))
			return list;

		int idx = symptomdbPath.indexOf(";");
		while (idx != -1) {
			String data = symptomdbPath.substring(0, idx);
			int i = data.indexOf(",");

			if (i != -1 && data.substring(i + 1).equals("1")) {

				String path = data.substring(0, i);
				IWorkspace workbench = ResourcesPlugin.getWorkspace();
				IPath filePath = new Path(path);
				IResource fres = workbench.getRoot().findMember(filePath);
				if (fres != null && fres.exists()) {
					list.put(PLATFORM_PREFIX + path, new Long(getTimestamp(fres)));
				}
			}

			symptomdbPath = symptomdbPath.substring(idx + 1);
			idx = symptomdbPath.indexOf(";");
		}

		int i = symptomdbPath.indexOf(",");
		if (i != -1 && symptomdbPath.substring(i + 1).equals("1")) {

			String path = symptomdbPath.substring(0, i);
			IWorkspace workbench = ResourcesPlugin.getWorkspace();
			IPath filePath = new Path(path);
			IResource fres = workbench.getRoot().findMember(filePath);
			if (fres != null && fres.exists()) {
				list.put(PLATFORM_PREFIX + path, new Long(getTimestamp(fres)));
			}
		}

		return list;
	}
    
    
    protected boolean databaseExists(String path) {
        if (path == null)
            return false;

        path = resourcePath(URI.createURI(path));
        IWorkspace workbench = ResourcesPlugin.getWorkspace();
        IResource res = workbench.getRoot().findMember(path);
        if (res == null || !res.exists())
            return false;

        return true;
    }
    
	protected String resourcePath(URI uri)
	{
	   String path = uri.path();
  
	   if(path.startsWith("/resource"))
		 return path.substring(9);
   	  
	   if(path.startsWith(PLATFORM_PREFIX))
		 return path.substring(18);

	   return path;  
	}
    
    public String analyze(Object selection){

		StringBuffer list = new StringBuffer();
		Iterator i = ((IStructuredSelection)selection).iterator();
		while(i.hasNext()){
			Object elem = i.next();
			if(elem instanceof CBEDefaultEvent){
				list.append(analyzeEvent((CBEDefaultEvent)elem));

			}
		}
		
		return list.toString();	    
    }
    
    protected String analyzeEvent(CBEDefaultEvent logRecord){
    	
		String symptomString = getSymptomStringFor(logRecord);
		
		return findMatchesFor(logRecord, symptomString);
        
    }
    
    protected String getSymptomStringFor(CBEDefaultEvent logRecord){

		String symptomString = logRecord.getExtensionName();
		
		if(logRecord instanceof CBECommonBaseEvent)
			symptomString = ((CBECommonBaseEvent)logRecord).getMsg();
			
		return 	symptomString;
    
    }
    
    private String findMatchesFor(CBEDefaultEvent logRecord, String symptomString){

		EList directivesList = ((CBEDefaultEvent) logRecord).getSymptoms();
        
		if (!directivesList.isEmpty())
			directivesList.clear();

	   Set keySet = _analysisEngine.keySet();
	   for (Iterator iter = keySet.iterator(); iter.hasNext();) {
		   AnalysisEngine engine = (AnalysisEngine) iter.next();			 
			Object[] directives = engine.analyze(new Incident(symptomString));

			for (int idx = 0; idx < directives.length; idx++) {
			 	
				logRecord.getSymptoms().add((SDBSymptom)directives[idx]);
			}
			 
		}

		((CBEDefaultEvent) logRecord).setAnalyzed(true);
		return symptomString;
    	
    
    }
     private long getTimestamp(IResource resource){

		long currentTimestamp = 0;
		IPath path = resource.getLocation();		
		if (path != null){
			currentTimestamp = path.toFile().lastModified();
		}		
		return currentTimestamp;
    	
    }

}

==> hyades.source_1.3.0_20040422_2239.zip: org/eclipse/hyades/log/ui/internal/views/LogAnalyzerImpl.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.log.ui.internal.views;

import java.util.*;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.emf.common.util.*;
import org.eclipse.hyades.analysis.engine.*;
import org.eclipse.hyades.log.ui.internal.*;
import org.eclipse.hyades.models.cbe.*;
import org.eclipse.hyades.models.internal.sdb.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.hyades.models.internal.sdb.loader.*;

public class LogAnalyzerImpl implements ILogAnalyzer {

	protected String errorMessage;
    protected Map _analysisEngine = new Hashtable();
    private final String PLATFORM_PREFIX = "platform:/resource";	
	/**
	 * Constructor for LogAnalyzerImpl.
	 */
	public LogAnalyzerImpl() {
		super();
	}

	/**
	 * @see org.eclipse.hyades.analysis.engine.ILogAnalyzer#errorMsg()
	 */
	public String errorMsg() {
		return errorMessage;
	}

	public void importDB(IProgressMonitor monitor)
	{
		//imports the DB since it is not yet within the workspace
		String absolutePath = "";
		try
		{
			String symptomdbPath = AnalysisEnginePlugin.getPlugin().getPreferenceStore().getString(AnalysisEngineConstants.SYMPTOM_DB_PATH);
			String pName = AnalysisEnginePlugin.getPlugin().getPreferenceStore().getString(AnalysisEngineConstants.PROJECT_NAME);
			String sdbFileName = AnalysisEnginePlugin.getPlugin().getPreferenceStore().getString(AnalysisEngineConstants.SDB_FILE_NAME);
			
			int i = symptomdbPath.indexOf(",");
			if (i != -1 && symptomdbPath.substring(i + 1, i + 2 ).equals("1")) {
				absolutePath = symptomdbPath.substring(0, i);
			}

			SDBLoader l = new SDBLoader();
			l.loadDatabase(new Path(sdbFileName).toString(),new Path(PLATFORM_PREFIX + "/" + absolutePath).toString() , monitor);
			
		}catch(Exception e){
		}
			
	}
	/**
	 * @see org.eclipse.hyades.analysis.engine.ILogAnalyzer#loadDatabase()
	 */
	public void loadDatabase() {
		
		errorMessage = null;

		Map list = getSymptomDBPathList();
		
		Set keySet = _analysisEngine.keySet();

		for (Iterator iter = keySet.iterator(); iter.hasNext();) {
			AnalysisEngine element = (AnalysisEngine) iter.next();
			if (!databaseExists(element.getSymptomDatabasePath())) {
				element.removeSymptomDatabase();
				iter.remove();
				continue;
			}			

			String path = element.getSymptomDatabasePath();			
			Long value = (Long)list.get(path);
			if(value==null){
				element.removeSymptomDatabase();
				iter.remove();			
			}else if((value!=null && value.longValue()!=((Long)_analysisEngine.get(element)).longValue())){				
				element.removeSymptomDatabase();
				iter.remove();			
			}else{
				list.remove(path);
			}
		}
		
		Set listSet = list.keySet();
		for (Iterator iter = listSet.iterator(); iter.hasNext();) {
			String path = (String) iter.next();
			_analysisEngine.put(AnalysisEngine.getInstance(path), list.get(path));			
		}
		
		if (_analysisEngine.size() == 0) {

			errorMessage = LogUIPlugin.getResourceString("STR_NO_SYMPTOM_DB_ERROR");

		}    
	}
		
	/**
	 * @see org.eclipse.hyades.analysis.engine.ILogAnalyzer#unloadDatabase()
	 */
	public void unloadDatabase() {

		Set analysisSet = _analysisEngine.keySet();
		for (Iterator iter = analysisSet.iterator(); iter.hasNext();) {
			AnalysisEngine engine = (AnalysisEngine) iter.next();
			engine.removeSymptomDatabase();

		}
		_analysisEngine.clear();			

	}

	protected Map getSymptomDBPathList() {
		Map list = new HashMap();
		String symptomdbPath = AnalysisEnginePlugin.getPlugin().getPreferenceStore().getString(AnalysisEngineConstants.SYMPTOM_DB_PATH);

		if (symptomdbPath == null || symptomdbPath.equals(""))
			return list;

		int idx = symptomdbPath.indexOf(";");
		while (idx != -1) {
			String data = symptomdbPath.substring(0, idx);
			int i = data.indexOf(",");

			if (i != -1 && data.substring(i + 1).equals("1")) {

				String path = data.substring(0, i);
				IWorkspace workbench = ResourcesPlugin.getWorkspace();
				IPath filePath = new Path(path);
				IResource fres = workbench.getRoot().findMember(filePath);
				if (fres != null && fres.exists()) {
					list.put(PLATFORM_PREFIX + path, new Long(getTimestamp(fres)));
				}
			}

			symptomdbPath = symptomdbPath.substring(idx + 1);
			idx = symptomdbPath.indexOf(";");
		}

		int i = symptomdbPath.indexOf(",");
		if (i != -1 && symptomdbPath.substring(i + 1).equals("1")) {

			String path = symptomdbPath.substring(0, i);
			IWorkspace workbench = ResourcesPlugin.getWorkspace();
			IPath filePath = new Path(path);
			IResource fres = workbench.getRoot().findMember(filePath);
			if (fres != null && fres.exists()) {
				list.put(PLATFORM_PREFIX + path, new Long(getTimestamp(fres)));
			}
		}

		return list;
	}
    
    
    protected boolean databaseExists(String path) {
        if (path == null)
            return false;

        path = resourcePath(URI.createURI(path));
        IWorkspace workbench = ResourcesPlugin.getWorkspace();
        IResource res = workbench.getRoot().findMember(path);
        if (res == null || !res.exists())
            return false;

        return true;
    }
    
	protected String resourcePath(URI uri)
	{
	   String path = uri.path();
  
	   if(path.startsWith("/resource"))
		 return path.substring(9);
   	  
	   if(path.startsWith(PLATFORM_PREFIX))
		 return path.substring(18);

	   return path;  
	}
    
    public String analyze(Object selection){

		StringBuffer list = new StringBuffer();
		Iterator i = ((IStructuredSelection)selection).iterator();
		while(i.hasNext()){
			Object elem = i.next();
			if(elem instanceof CBEDefaultEvent){
				list.append(analyzeEvent((CBEDefaultEvent)elem));

			}
		}
		
		return list.toString();	    
    }
    
    protected String analyzeEvent(CBEDefaultEvent logRecord){
    	
		String symptomString = getSymptomStringFor(logRecord);
		
		return findMatchesFor(logRecord, symptomString);
        
    }
    
    protected String getSymptomStringFor(CBEDefaultEvent logRecord){

		String symptomString = logRecord.getExtensionName();
		
		if(logRecord instanceof CBECommonBaseEvent)
			symptomString = ((CBECommonBaseEvent)logRecord).getMsg();
			
		return 	symptomString;
    
    }
    
    private String findMatchesFor(CBEDefaultEvent logRecord, String symptomString){

		EList directivesList = ((CBEDefaultEvent) logRecord).getSymptoms();
        
		if (!directivesList.isEmpty())
			directivesList.clear();

	   Set keySet = _analysisEngine.keySet();
	   for (Iterator iter = keySet.iterator(); iter.hasNext();) {
		   AnalysisEngine engine = (AnalysisEngine) iter.next();			 
			Object[] directives = engine.analyze(new Incident(symptomString));

			for (int idx = 0; idx < directives.length; idx++) {
			 	
				logRecord.getSymptoms().add((SDBSymptom)directives[idx]);
			}
			 
		}

		((CBEDefaultEvent) logRecord).setAnalyzed(true);
		return symptomString;
    	
    
    }
     private long getTimestamp(IResource resource){

		long currentTimestamp = 0;
		IPath path = resource.getLocation();		
		if (path != null){
			currentTimestamp = path.toFile().lastModified();
		}		
		return currentTimestamp;
    	
    }

}
